
function [a,indx]=OMP(x,Dict,T,E2M)
%------------------------------------------------------------------------
% Function for Orthogonal Matching Pursuit (OMP) 
% Inputs:
%   x: vector of the selected frame 
%   Dict: the dictionary 
%   T: maximum number of coefficients 
%   E2M: Tolerance
% Outputs:
%   a: vector of the output coefficients
%   indx: indices of the selected atoms from Dict
% -------------------------------------------------------------------------
% Function implemented in the Audio Inpainting Toolbox which can be downloaded 
% from: http://www.small-project.eu/software-data/AudioInpaintingToolbox.zip/view.html
% The Function was modified by: Toumi.I
% Mai 2017
% ------------------------------------------------------------------------

Omega=size(Dict,2);
a=zeros(Omega,1);
residual=x;
maxNumCoef = T;
indx = [];
currResNorm2 = E2M*2; % set a value above the threshold in order to have/force at least one loop executed
j = 0;

while  j < maxNumCoef && abs(currResNorm2)>E2M
    
    j = j+1;
    
    % Atom selection
    proj=Dict'*residual;
    [dum pos] = max(abs(proj));
    
    % Update support
    indx(j)=pos;
    
    % Update solution
    Dictj = Dict(:,indx(1:j));
    a(indx(1:j))=pinv(Dictj)*x;
    
    % Update residue
    residual=x-Dict(:,indx(1:j))*a(indx(1:j));
    
    % Compute error
    currResNorm2=sum(residual.^2);

end
