
function [A,indx]=SOMP_Inpainting(X,D,Mask,T,E2M)
% ------------------------------------------------------------------
% S-OMP Algorithm function (Simulatenous Orthogonal Matching Pursuit)
% Inpainting case 
% Inputs:
%   X: matrix containing similar frames
%   D: the dictionary not normalized
%   Mask: The mask ( 1 for masked samples and 0 else )
%   T: maximum number of coefficients 
%   E2M: Tolerance
% Outputs:
%   A: matrix of output coefficients
%   indx: indices of the selected atoms from Dict
%                       
% Implemented by: Toumi.I
% October 2017
% --------------------------------------------------------------------

% Initialization

M=size(X,2);                % Number of similar frames 
Omega=size(D,2);            % Number of atoms in the dictionary

A=zeros(Omega,M);           % Matrix of coefficients for all the similar frames
proj=zeros(Omega,M);        % Projection matrix
R = X;                      % Residual matrix

maxNumCoef = T;             % Maximum number of coefficients for S-OMP
indx = [];                  % Indexes of selected atoms
currResNorm2 = E2M*2*M;     % set a value above the threshold in order to have at least one loop executed
iter = 0;
XE2M = E2M*M;


while iter < maxNumCoef && currResNorm2>XE2M
    
    iter = iter+1;
    
    % Greedy selection
    for m = 1:M
        MObs = find(~Mask(:,m));            % Select observed samples 
        DM = D(MObs,:);                     % # All the dictionary atoms 
        WM = 1./sqrt(sum(abs(DM).^2));
        DM = DM.*repmat(WM,size(DM,1),1);   % Normalization 
        proj(:,m) = DM'*R(MObs,m);
    end
    
    % Atom selection
%     [~,pos] = max(sum(proj.^2,2)); 

    [~,pos] = max(sum(abs(proj),2));    % Selection method ( like S-OMP # Tropp ) 
    
    % Update support
    indx(iter)=pos;

    for m=1:M   % To get coefficients of all the selected frames
        MObs = find(~Mask(:,m));
        DMj = D(MObs,indx(1:iter));
        WMj = 1./sqrt(sum(abs(DMj).^2));
        DMj = DMj.*repmat(WMj,size(DMj,1),1);
        % Update Solution
        A(indx(1:iter),m) = pinv(DMj)*X(MObs,m);
        
        % Update residue
        R(MObs,m)=X(MObs,m)-DMj*A(indx(1:iter),m);
    end
    
    % Update error
    currResNorm2=sum(sum(abs(R).^2));
end
end


