function y = FFTSimSignal(x0,x,h,method)

% -----------------------------------------------------------------------
% The function compute the similarity measure between a fixed frame x0 from
% the set of frames x using different similarity measures in the FFT domain
% Output: y: The distance measure among all the frames compared to x0
% Inputs: x0:The reference frame 
%         x: the signal 
%         h: hop size for frames 
%         method: the similarity measure to be used:
%                 --> CosDist: it measures the cosinus distance between 
%                     FFT(x0) and FFT (x)
%                 --> CosDistAbs: it measures the cosinus distance between 
%                     |FFT(x0)| and |FFT (x)|
%                 --> ISDist: it measures the Itakura Saito distance between 
%                     FFT(x0) and FFT(x)
% By: Toumi.I
% Mai 2017 
% Qarma Team - LIF 
% ------------------------------------------------------------------------


N = size(x0,1);

% Frames selection and calculation   
Nx = size(x,1);
nList = 1:h:Nx-N+1;
ln= length(nList);
frames = zeros(N,ln);

for k = 1:ln
  n = nList(k);
  frames(:,k) = x(n+(0:N-1));
end

y= zeros(length(nList),1);

% FFT domain
Xfft=fft(frames,N);
Xfft0=fft(x0,N);

% selecting only coefficients of positive frequencies
% Xfft=Xfft(1:N/2,:);
% Xfft0=Xfft0(1:N/2);

% Similarity measure computation 
if strcmp(method,'CosDist'),
    for k=1:ln
        Xfftn=Xfft(:,k)/norm(Xfft(:,k));        % normalization
        Xfft0n=Xfft0./norm(Xfft0);              % normalization
        y(k)= abs(sum(conj(Xfft0n).'*Xfftn));   % scalar product
    end
else if strcmp(method,'ISDist'),
        for k=1:ln
            Xfftn=abs(Xfft(:,k)./norm(Xfft(:,k)));   % normalization
            Xfft0n=abs(Xfft0./norm(Xfft0));          % normalization
            y(k)= (1/N).*sum(Xfft0n./Xfftn-log(Xfft0n./Xfftn)-1);   %ISDist
        end
        % must normalize by the max after 

    else if strcmp(method,'CosDistAbs'),
            for k=1:ln
                Xfftn=abs(Xfft(:,k))./norm(abs(Xfft(:,k)));     % normalization
                Xfft0n=abs(Xfft0)./norm(abs(Xfft0));            % normalization
                y(k)= abs(sum(conj(Xfft0n).'*Xfftn));           % scalar product
            end
        end
    end
end

