% ------------------------------------------------------------------------
% Function to plot the average SNR variation vs. number of selected frames 
% LIF 2017
% -------------------------------------------------------------------------

clear All
close All

setpath;

soundname = [{'Speech'} {'Piano'}];
NbFram = [1, 2, 4, 8, 16, 32];  % Number of similar frames used for the joint sparsity

lf = length(soundname);
ln = length(NbFram);

%% Load results

SNR_ALL_OMP = zeros(lf,ln);
SNR_ALL_OMPG = zeros(lf,ln);
SNR_ALL_SOMP = zeros(lf,ln);

SNR_MISS_OMP = zeros(lf,ln);
SNR_MISS_OMPG = zeros(lf,ln);
SNR_MISS_SOMP = zeros(lf,ln);

for i = 1:lf
    for j = 1:ln
        filename=['Inpainting_' soundname{i} '_RandomHoles_50percent_NbFram' num2str(NbFram(j))];
        
        load(filename,'SNR_SOMP_allsamples','SNR_SOMP_missingsamples','SNR_OMP_allsamples',...
            'SNR_OMPG_allsamples','SNR_OMP_missingsamples','SNR_OMPG_missingsamples');
        
        SNR_ALL_OMP(i,j) = SNR_OMP_allsamples;
        SNR_MISS_OMP(i,j) = SNR_OMP_missingsamples;
        
        SNR_ALL_OMPG(i,j) = SNR_OMPG_allsamples;
        SNR_MISS_OMPG(i,j) = SNR_OMPG_missingsamples;
        
        SNR_ALL_SOMP(i,j) = SNR_SOMP_allsamples;
        SNR_MISS_SOMP(i,j) = SNR_SOMP_missingsamples;
    end
end


%% SNR Average


Mean_SNR_ALL_OMP = mean(SNR_ALL_OMP,2);
Mean_SNR_MISS_OMP = mean(SNR_MISS_OMP,2);

Mean_SNR_ALL_OMPG = mean(SNR_ALL_OMPG,2);
Mean_SNR_MISS_OMPG = mean(SNR_MISS_OMPG,2);


%% Plot figure

figure; % SNR for speech
set(gcf,'Color',[1,1,1]);
subplot(121);hold on; 
plot(NbFram,SNR_MISS_SOMP(1,:),'-rs','LineWidth',1.2); 
plot(NbFram,repmat(Mean_SNR_MISS_OMP(1),1,6),'-*','LineWidth',1.2); 
plot(NbFram,repmat(Mean_SNR_MISS_OMPG(1),1,6),'-kv','LineWidth',1.2); legend('S-OMP F','OMP F','OMP G');
xlabel('Number of selected frames','fontsize',12,'fontWeight','bold');
ylabel('Average SNRm (dB)','fontsize',12,'fontWeight','bold'); ylim([10,16]); axis square; grid on 

subplot(122);hold on;  % SNR for Piano
plot(NbFram,SNR_MISS_SOMP(2,:),'-rs','LineWidth',1.2); 
plot(NbFram,repmat(Mean_SNR_MISS_OMP(2),1,6),'-*','LineWidth',1.2); 
plot(NbFram,repmat(Mean_SNR_MISS_OMPG(2),1,6),'-kv','LineWidth',1.2); legend('S-OMP F','OMP F','OMP G');
xlabel('Number of selected frames','fontsize',12,'fontWeight','bold'); 
ylabel('Average SNRm (dB)','fontsize',12,'fontWeight','bold'); ylim([10,20]); axis square; grid on
