% -------------------------------------------------------------------------
% Plot similarity Map for an audio signal and for different similarity measures
% By: Toumi.I
% LIF, 2017
% -------------------------------------------------------------------------

setpath;

clear all
close all

%% Read test signal
[sig,~]= audioread('Audio/Speech.wav');

N = 256;                            % Frame length
ls = size(sig,1);                   % Signal length

xMax = 0.9999;                      % Max value
sig = sig/max(abs(sig))*xMax;       % Normalization

h=round(N/4);                       % Hop size 
n0List = 1:h:ls-N+1;                % List of the Frames start's indexes 
L0=length(n0List);                  % Number of the frames

NbFram=round(L0/8);                 % Number of similar frames to be selected 

%% load optimized coeffs with OMP

load('DecompFFT_speech_clip1_T064_hop064','Xfft');

% Consider only coefficients corresponding to positive frequencies 
Xfft1=Xfft(1:N,:);

% Put to 1 all the non zero information
ind=find(Xfft1);
Xfftb=Xfft1;
Xfftb(ind)=1;

%% Similar set selection

Gamma_corr=zeros(L0,L0);        % Similarity Matrix - correlation
Gamma_m=zeros(L0,L0);           % Similarity Matrix - Spectral cosine similarity
D_IS=zeros(L0,L0);              % Distance Matrix - Itakura Saito
D_ham=zeros(L0,L0);             % Distance Matrix - Hamming 

for n0=1:L0  % Each frame number
    
    kn0 = n0List(n0);
    s0 = sig(kn0+(0:N-1));  

    Gamma_corr(:,n0)=FFTSimSignal(s0,sig,h,'CosDist');
    Gamma_m(:,n0)=FFTSimSignal(s0,sig,h,'CosDistAbs');
    D_IS(:,n0)=FFTSimSignal(s0,sig,h,'ISDist');
    
    % Hamming distance - selection 
    Xfft0=Xfftb(:,n0);
    for k=1:L0
        D_ham(k,n0)=dist_Ham(abs(Xfft0),abs(Xfftb(:,k)));
    end
    fprintf('End computation for frame n: %d / %d \n',n0,L0);
end

D_IS = D_IS./max(max(D_IS)); % Normalization of the IS distance matrix
Gamma_IS = 1-D_IS;

D_ham = D_ham./max(max(D_ham)); % Normalization of the hamming distance matrix
Gamma_ham = 1-D_ham;

%% The measures are sorted increasingly from min to max 

[Acorr,Bcorr]=sort(Gamma_corr);
[Am,Bm]=sort(Gamma_m);
[AIS,BIS]=sort(Gamma_IS);
[Aham,Bham]=sort(Gamma_ham);


%%  Save only the values of similar frames to plot them

for i=1:L0
    sf = Gamma_corr(:,i);
    ind = Bcorr(1:end-NbFram,i);
    sf(ind) = 0;
    Gamma_corr(:,i) = sf;
end


for i=1:L0
    sf = Gamma_m(:,i);
    ind = Bm(1:end-NbFram,i);
    sf(ind) = 0;
    Gamma_m(:,i) = sf;
end


for i=1:L0
    sf = Gamma_IS(:,i);
    ind = BIS(1:end-NbFram,i);
    sf(ind) = 0;
    Gamma_IS(:,i) = sf;
end

%% Plot Map of similar frames 
figure;
set(gcf,'Color',[1,1,1]);
subplot(221); imagesc(log10(Gamma_ham(150:250,150:250)));  axis square
title('Hamming Similarity','fontsize',16,'fontWeight','bold')
subplot(222); imagesc(log10(Gamma_corr(150:250,150:250)));  axis square
title('Correlation Similarity','fontsize',16,'fontWeight','bold');
subplot(223); imagesc(log10(Gamma_m(150:250,150:250)));  axis square
title('Spectral Cosine Similarity','fontsize',16,'fontWeight','bold');
subplot(224); imagesc(log10(Gamma_IS(150:250,150:250))); 
title('IS Similarity','fontsize',16,'fontWeight','bold');  axis square
