% -------------------------------------------------------------------------
% Compute average intersection between the sets of the similar frames for the 
% different similarity measures and the reference set computed with hamming 
% distance 
% Inpainting case
% By: Ichrak Toumi
% LIF, 2017
% -------------------------------------------------------------------------

clear all
close all

setpath;
%% Read test signal

[SoundName, SoundPath] = uigetfile(('*.wav'),'Select the audio file');
myAudio=fullfile(SoundPath, SoundName);
[sig,Fs] = audioread(myAudio);


N = 256;
ls = size(sig,1);

xMax = 0.9999;
sig = sig/max(abs(sig))*xMax;

h=round(N/4);
n0List = 1:h:ls-N+1;
L0=length(n0List);

NbFram=4;

%% load FFT coeffs with OMP

[FileName,FilePath] = uigetfile(('*.mat'),'Decomposition matfile');
myMatfile=fullfile(FilePath, FileName);
load(myMatfile);

%% Put to 1 all the non zero information  (hamming distance)

ind=find(Xfft);
Xfftb=Xfft;
Xfftb(ind)=1;

%% Intersection between similar sets and the reference set

intercorrAll = zeros(L0,1);
intermAll = zeros(L0,1);
interISAll = zeros(L0,1);

for n0=1:L0
    
    kn0 = n0List(n0);               % Selection of the frame index
    s0 = sig(kn0+(0:N-1));          % Selection of the frame 

    % Similarity measures
    gamma_corr=FFTSimSignal(s0,sig,h,'CosDist');
    gamma_m=FFTSimSignal(s0,sig,h,'CosDistAbs');
    d_IS=FFTSimSignal(s0,sig,h,'ISDist');
    
    % Sets of similar frames indexes 
    [Acorr,Bcorr]=sort(1-gamma_corr);
    ScorrList=Bcorr(1:NbFram);
    
    [Am,Bm]=sort(1-gamma_m);
    SmList=Bm(1:NbFram);    
    
    [AIS,BIS]=sort(d_IS);
    SISList=BIS(1:NbFram);

    % Searching the similar frames for the selected frame
    Xfft0=Xfftb(:,n0);
    dist_ham=zeros(L0,1);
    
    for k=1:L0
        % Hamming distance - selection 
        dist_ham(k)=dist_Ham(abs(Xfftb(:,k)),abs(Xfft0));
    end
    
    % Reference set 
    [dist_s,ind_s]=sort(dist_ham);
    Sn0=sort(ind_s(1:NbFram));
    
    % Compute the length of intersection between the sets
    intercorrAll(n0)=length(intersect(ScorrList,Sn0));
    intermAll(n0)=length(intersect(SmList,Sn0));
    interISAll(n0)=length(intersect(SISList,Sn0));
    
    fprintf('end frame n: %d / %d \n',n0, L0);
end

% Percentage of intersection between the reference set and the set of each
% measure

intercorr=mean(intercorrAll)*(100/NbFram);
interm=mean(intermAll)*(100/NbFram);
interIS=mean(interISAll)*(100/NbFram);

% Save results
File_out = ['SimIntersection_' SoundName(1:length(SoundName)-4) '_RandomHoles_' num2str(percentage*100) 'percent'];

save(File_out,'intercorr','interm','interIS','h','percentage','NbFram');
